@@ -31,7 +31,9 @@ const kpActions = {
     GET_DATABASE_GROUPS: 'get-database-groups',
     CREATE_NEW_GROUP: 'create-new-group',
     GET_TOTP: 'get-totp',
-    REQUEST_AUTOTYPE: 'request-autotype'
+    REQUEST_AUTOTYPE: 'request-autotype',
+    PASSKEYS_REGISTER: 'passkeys-register',
+    PASSKEYS_GET: 'passkeys-get'
 };
 
 browser.storage.local.get({ 'latestKeePassXC': { 'version': '', 'lastChecked': null }, 'keyRing': {} }).then((item) => {
@@ -117,23 +119,15 @@ keepass.retrieveCredentials = async function(tab, args = []) {
         }
 
         let entries = [];
-        const keys = [];
         const kpAction = kpActions.GET_LOGINS;
         const nonce = keepassClient.getNonce();
         const [ dbid ] = keepass.getCryptoKey();
 
-        for (const keyHash in keepass.keyRing) {
-            keys.push({
-                id: keepass.keyRing[keyHash].id,
-                key: keepass.keyRing[keyHash].key
-            });
-        }
-
         const messageData = {
             action: kpAction,
             id: dbid,
             url: url,
-            keys: keys
+            keys: keepass.getCryptoKeys()
         };
 
         if (submiturl) {
@@ -168,18 +162,18 @@ keepass.retrieveCredentials = async function(tab, args = []) {
 
 keepass.generatePassword = async function(tab) {
     if (!keepass.isConnected) {
-        return [];
+        return undefined;
     }
 
     try {
         const taResponse = await keepass.testAssociation(tab);
         if (!taResponse) {
             browserAction.showDefault(tab);
-            return [];
+            return '';
         }
 
         if (!keepass.compareVersion(keepass.requiredKeePassXC, keepass.currentKeePassXC)) {
-            return [];
+            return '';
         }
 
         let password;
@@ -204,7 +198,7 @@ keepass.generatePassword = async function(tab) {
         return password;
     } catch (err) {
         logError(`generatePassword failed: ${err}`);
-        return [];
+        return undefined;
     }
 };
 
@@ -241,7 +235,7 @@ keepass.associate = async function(tab) {
             keepass.associated.value = true;
             keepass.associated.hash = response.hash || 0;
 
-            browserAction.show(tab);
+            browserAction.showDefault(tab);
             return AssociatedAction.NEW_ASSOCIATION;
         }
 
@@ -268,10 +262,6 @@ keepass.testAssociation = async function(tab, args = []) {
             return false;
         }
 
-        if (keepass.isAssociated()) {
-            return true;
-        }
-
         if (!keepass.serverPublicKey) {
             if (tab && page.tabs[tab.id]) {
                 keepass.handleError(tab, kpErrors.PUBLIC_KEY_NOT_FOUND);
@@ -387,8 +377,9 @@ keepass.getDatabaseHash = async function(tab, args = []) {
 
         keepass.databaseHash = '';
         keepass.isDatabaseClosed = true;
-        if (response.message && response.message === '') {
+        if ((response.message && response.message === '') || response.errorCode === kpErrors.TIMEOUT_OR_NOT_CONNECTED) {
             keepass.isKeePassXCAvailable = false;
+            keepass.isConnected = false;
             keepass.handleError(tab, kpErrors.TIMEOUT_OR_NOT_CONNECTED);
         } else {
             keepass.handleError(tab, response.errorCode, response.error);
@@ -453,7 +444,6 @@ keepass.lockDatabase = async function(tab) {
         action: kpAction
     };
 
-
     try {
         const response = await keepassClient.sendMessage(kpAction, tab, messageData, nonce);
         if (response) {
@@ -608,6 +598,74 @@ keepass.requestAutotype = async function(tab, args = []) {
     }
 };
 
+keepass.passkeysRegister = async function(tab, args = []) {
+    try {
+        const taResponse = await keepass.testAssociation(tab, [ false ]);
+        if (!taResponse || !keepass.isConnected || args.length < 2) {
+            browserAction.showDefault(tab);
+            return [];
+        }
+
+        const kpAction = kpActions.PASSKEYS_REGISTER;
+        const nonce = keepassClient.getNonce();
+
+        // Parse publicKey
+        const publicKey = args[0];
+        const origin = args[1];
+
+        const messageData = {
+            action: kpAction,
+            publicKey: JSON.parse(JSON.stringify(publicKey)),
+            origin: origin,
+            keys: keepass.getCryptoKeys()
+        };
+
+        const response = await keepassClient.sendMessage(kpAction, tab, messageData, nonce);
+        if (response) {
+            return response;
+        }
+
+        browserAction.showDefault(tab);
+        return [];
+    } catch (err) {
+        logError(`passkeysRegister failed: ${err}`);
+        return [];
+    }
+};
+
+keepass.passkeysGet = async function(tab, args = []) {
+    try {
+        const taResponse = await keepass.testAssociation(tab, [ false ]);
+        if (!taResponse || !keepass.isConnected || args.length < 2) {
+            browserAction.showDefault(tab);
+            return [];
+        }
+
+        const kpAction = kpActions.PASSKEYS_GET;
+        const nonce = keepassClient.getNonce();
+        const publicKey = args[0];
+        const origin = args[1];
+
+        const messageData = {
+            action: kpAction,
+            publicKey: JSON.parse(JSON.stringify(publicKey)),
+            origin: origin,
+            keys: keepass.getCryptoKeys()
+        };
+
+        const response = await keepassClient.sendMessage(kpAction, tab, messageData, nonce);
+        if (response) {
+            return response;
+        }
+
+        browserAction.showDefault(tab);
+        return [];
+    } catch (err) {
+        logError(`passkeysGet failed: ${err}`);
+        return [];
+    }
+};
+
 //--------------------------------------------------------------------------
 // Keyring
 //--------------------------------------------------------------------------
@@ -707,6 +765,19 @@ keepass.setCryptoKey = function(id, key) {
     keepass.saveKey(keepass.databaseHash, id, key);
 };
 
+keepass.getCryptoKeys = function() {
+    const keys = [];
+
+    for (const keyHash in keepass.keyRing) {
+        keys.push({
+            id: keepass.keyRing[keyHash].id,
+            key: keepass.keyRing[keyHash].key
+        });
+    }
+
+    return keys;
+};
+
 //--------------------------------------------------------------------------
 // Connection
 //--------------------------------------------------------------------------
@@ -759,6 +830,10 @@ keepass.reconnect = async function(tab, connectionTimeout) {
 // Utils
 //--------------------------------------------------------------------------
 
+keepass.getErrorMessage = async function(tab, errorCode) {
+    return kpErrors.getError(errorCode);
+};
+
 keepass.generateNewKeyPair = function() {
     keepass.keyPair = nacl.box.keyPair();
 };
