@@ -18,22 +18,25 @@ const defaultSettings = {
     defaultGroup: '',
     defaultGroupAlwaysAsk: false,
     downloadFaviconAfterSave: false,
-    redirectAllowance: 3,
+    passkeys: false,
+    passkeysFallback: true,
+    redirectAllowance: 1,
     saveDomainOnly: true,
     showGettingStartedGuideAlert: true,
     showTroubleshootingGuideAlert: true,
     showLoginFormIcon: true,
     showLoginNotifications: true,
     showNotifications: true,
+    useMonochromeToolbarIcon: false,
     showOTPIcon: true,
     useObserver: true,
     usePredefinedSites: true,
-    usePasswordGeneratorIcons: false
+    usePasswordGeneratorIcons: false,
 };
 
 const AUTO_SUBMIT_TIMEOUT = 5000;
 
-var page = {};
+const page = {};
 page.autoSubmitPerformed = false;
 page.attributeMenuItemIds = [];
 page.blockedTabs = [];
@@ -196,17 +199,25 @@ page.createTabEntry = function(tabId) {
 // Page reload or tab switch clears the cache.
 // If the retrieval is forced (from Credential Banner), get new credentials normally.
 page.retrieveCredentials = async function(tab, args = []) {
+    if (!tab?.active) {
+        return [];
+    }
+
     const [ url, submitUrl, force ] = args;
     if (page.tabs[tab.id]?.credentials.length > 0 && !force) {
         return page.tabs[tab.id].credentials;
     }
 
-    // Ignore duplicate requests
-    if (page.currentRequest.url === url && page.currentRequest.submitUrl === submitUrl && !force) {
+    // Ignore duplicate requests from the same tab
+    if (page.currentRequest.url === url
+        && page.currentRequest.submitUrl === submitUrl
+        && page.currentRequest.tabId === tab.id
+        && !force) {
         return [];
     } else {
         page.currentRequest.url = url;
         page.currentRequest.submitUrl = submitUrl;
+        page.currentRequest.tabId = tab.id;
     }
 
     const credentials = await keepass.retrieveCredentials(tab, args);
@@ -301,8 +312,8 @@ page.updateContextMenu = async function(tab, credentials) {
             // Show username inside [] if there are KPH attributes inside multiple credentials
             const attributeName = Object.keys(attribute)[0].slice(5);
             const finalName = credentials.length > 1
-                            ? `[${cred.login}] ${attributeName}`
-                            : attributeName;
+                ? `[${cred.login}] ${attributeName}`
+                : attributeName;
 
             page.attributeMenuItemIds.push(createContextMenuItem({
                 action: 'fill_attribute',
@@ -314,6 +325,10 @@ page.updateContextMenu = async function(tab, credentials) {
     }
 };
 
+page.updatePopup = function(tab) {
+    browserAction.showDefault(tab);
+};
+
 const createContextMenuItem = function({ action, args, ...options }) {
     return browser.contextMenus.create({
         contexts: menuContexts,
