@@ -262,8 +262,8 @@ kpxc.initCombinations = async function(inputs = []) {
     }
 
     const combinations = isCustomLoginFieldsUsed
-                       ? await kpxcFields.useCustomLoginFields()
-                       : await kpxcFields.getAllCombinations(inputs);
+        ? await kpxcFields.useCustomLoginFields()
+        : await kpxcFields.getAllCombinations(inputs);
     if (!combinations || combinations.length === 0) {
         if (isCustomLoginFieldsUsed) {
             kpxcUI.createNotification('warning', tr('optionsCustomFieldsNotFound'));
@@ -343,8 +343,8 @@ kpxc.initLoginPopup = function() {
     // Returns a login item with additional information for sorting
     const getLoginItem = function(credential, withGroup, loginId) {
         const title = credential.name.length < MAX_AUTOCOMPLETE_NAME_LEN
-                   ? credential.name
-                   : credential.name.substr(0, MAX_AUTOCOMPLETE_NAME_LEN) + '…';
+            ? credential.name
+            : credential.name.substr(0, MAX_AUTOCOMPLETE_NAME_LEN) + '…';
         const group = (withGroup && credential.group) ? `[${credential.group}] ` : '';
         const visibleLogin = (credential.login.length > 0) ? credential.login : tr('credentialsNoUsername');
         let text = `${group}${title} (${visibleLogin})`;
@@ -546,22 +546,27 @@ kpxc.rememberCredentials = async function(usernameValue, passwordValue, urlValue
 
 // Save credentials triggered fron the context menu
 kpxc.rememberCredentialsFromContextMenu = async function() {
+    if (kpxc.databaseState === DatabaseState.LOCKED) {
+        kpxcUI.createNotification('error', tr('rememberErrorDatabaseClosed'));
+        return;
+    }
+
     const el = document.activeElement;
     if (el.nodeName !== 'INPUT') {
         return;
     }
 
-    const type = el.getAttribute('type');
-    const combination = await kpxcFields.getCombination(el, (type === 'password' ? type : 'username'));
+    const combination = await kpxcFields.getCombination(el);
     if (!combination) {
         logDebug('Error: No combination found.');
         return;
     }
 
-    const usernameValue = combination.username ? combination.username.value : '';
-    const passwordValue = combination.password ? combination.password.value : '';
+    const usernameValue = combination.username?.value ?? '';
+    const passwordValue = combination.password?.value ?? '';
 
-    const result = await kpxc.rememberCredentials(usernameValue, passwordValue, undefined, undefined, kpxc.settings.showLoginNotifications);
+    const result = await kpxc.rememberCredentials(usernameValue, passwordValue, undefined, undefined,
+        kpxc.settings.showLoginNotifications);
     if (result === undefined) {
         kpxcUI.createNotification('error', tr('rememberNoPassword'));
         return;
@@ -575,11 +580,17 @@ kpxc.rememberCredentialsFromContextMenu = async function() {
 // The basic function for retrieving credentials from KeePassXC.
 // Credential Banner can force the retrieval for reloading new/modified credentials.
 kpxc.retrieveCredentials = async function(force = false) {
+    if (!isIframeAllowed()) {
+        return [];
+    }
+
     kpxc.url = document.location.href;
     kpxc.submitUrl = kpxc.getFormActionUrl(kpxc.combinations[0]);
 
     if (kpxc.settings.autoRetrieveCredentials && kpxc.url && kpxc.submitUrl) {
-        await kpxc.retrieveCredentialsCallback(await sendMessage('retrieve_credentials', [ kpxc.url, kpxc.submitUrl, force ]));
+        await kpxc.retrieveCredentialsCallback(
+            await sendMessage('retrieve_credentials', [ kpxc.url, kpxc.submitUrl, force ]),
+        );
     }
 };
 
@@ -608,11 +619,7 @@ kpxc.retrieveCredentialsCallback = async function(credentials) {
 // If credentials are not received, request them again
 kpxc.receiveCredentialsIfNecessary = async function() {
     if (kpxc.credentials.length === 0 && !_called.retrieveCredentials) {
-        // Check for Cross-domain security error when inspecting window.top.location.href. We should ignore these requests.
-        try {
-            const currentLocation = window.top.location.href;
-        } catch (err) {
-            logDebug('Error: Credential request ignored from another domain: ', window.self.location.host);
+        if (!isIframeAllowed()) {
             return [];
         }
 
@@ -627,7 +634,8 @@ kpxc.receiveCredentialsIfNecessary = async function() {
             return [];
         }
 
-        // If the database was locked, this is scope never met. In these cases the response is met at kpxc.detectDatabaseChange
+        // If the database was locked, this is scope never met.
+        // In these cases the response is met at kpxc.detectDatabaseChange
         await sendMessage('page_set_manual_fill', ManualFill.NONE);
         await kpxc.retrieveCredentialsCallback(credentials);
 
@@ -667,12 +675,23 @@ kpxc.setValueWithChange = function(field, value, forced = false) {
         return;
     }
 
+    const dispatchLegacyEvent = function(elem, eventName) {
+        const legacyEvent = elem.ownerDocument.createEvent('Event');
+        legacyEvent.initEvent(eventName, true, false);
+        elem.dispatchEvent(legacyEvent);
+    };
+
+    field.focus();
+    field.dispatchEvent(new KeyboardEvent('keydown', { bubbles: true, cancelable: false }));
+    field.dispatchEvent(new KeyboardEvent('keypress', { bubbles: true, cancelable: false }));
+    field.dispatchEvent(new KeyboardEvent('keyup', { bubbles: true, cancelable: false }));
+    field.dispatchEvent(new Event('input', { bubbles: true, cancelable: false }));
+    field.dispatchEvent(new Event('change', { bubbles: true, cancelable: false }));
     field.value = value;
-    field.dispatchEvent(new Event('input', { bubbles: true }));
-    field.dispatchEvent(new Event('change', { bubbles: true }));
-    field.dispatchEvent(new KeyboardEvent('keydown', { bubbles: true, cancelable: false, key: '', char: '' }));
-    field.dispatchEvent(new KeyboardEvent('keypress', { bubbles: true, cancelable: false, key: '', char: '' }));
-    field.dispatchEvent(new KeyboardEvent('keyup', { bubbles: true, cancelable: false, key: '', char: '' }));
+
+    // Some pages will not accept the value change without dispatching events directly to the document
+    dispatchLegacyEvent(field, 'input');
+    dispatchLegacyEvent(field, 'change');
 };
 
 // Returns true if site is ignored
@@ -768,6 +787,44 @@ kpxc.updateTOTPList = async function() {
     return [];
 };
 
+// Apply a script to the page for intercepting Passkeys (WebAuthn) requests
+kpxc.enablePasskeys = function() {
+    const passkeys = document.createElement('script');
+    passkeys.src = browser.runtime.getURL('content/passkeys.js');
+    document.documentElement.appendChild(passkeys);
+
+    document.addEventListener('kpxc-passkeys-request', async (ev) => {
+        if (ev.detail.action === 'passkeys_create') {
+            const publicKey = kpxcPasskeysUtils.buildCredentialCreationOptions(ev.detail.publicKey);
+            logDebug('publicKey', publicKey);
+
+            const ret = await sendMessage('passkeys_register', [ publicKey, window.location.origin ]);
+            if (ret) {
+                if (ret.response && ret.response.errorCode) {
+                    const errorMessage = await sendMessage('get_error_message', ret.response.errorCode);
+                    kpxcUI.createNotification('error', errorMessage);
+                }
+
+                const responsePublicKey = kpxcPasskeysUtils.parsePublicKeyCredential(ret.response);
+                kpxcPasskeysUtils.sendPasskeysResponse(responsePublicKey);
+            }
+        } else if (ev.detail.action === 'passkeys_get') {
+            const publicKey = kpxcPasskeysUtils.buildCredentialRequestOptions(ev.detail.publicKey);
+            logDebug('publicKey', publicKey);
+
+            const ret = await sendMessage('passkeys_get', [ publicKey, window.location.origin ]);
+            if (ret) {
+                if (ret.response && ret.response.errorCode) {
+                    const errorMessage = await sendMessage('get_error_message', ret.response.errorCode);
+                    kpxcUI.createNotification('error', errorMessage);
+                }
+
+                const responsePublicKey = kpxcPasskeysUtils.parseGetPublicKeyCredential(ret.response);
+                kpxcPasskeysUtils.sendPasskeysResponse(responsePublicKey);
+            }
+        }
+    });
+};
 
 /**
  * Content script initialization.
@@ -787,6 +844,10 @@ const initContentScript = async function() {
             return;
         }
 
+        if (kpxc.settings.passkeys) {
+            kpxc.enablePasskeys();
+        }
+
         await kpxc.updateDatabaseState();
         await kpxc.initCredentialFields();
 
@@ -906,3 +967,14 @@ kpxc.reconnect = async function() {
     }
     return true;
 };
+
+// Check for Cross-domain security error when inspecting window.top.location.href
+const isIframeAllowed = function() {
+    try {
+        const currentLocation = window.top.location.href;
+        return true;
+    } catch (err) {
+        logDebug(`Error: Credential request ignored from another domain: ${window.self.location.host}`);
+        return false;
+    }
+};
