@@ -24,7 +24,7 @@ const acceptedParents = [
     '.mfa-verify',
 ];
 
-var kpxcTOTPIcons = {};
+const kpxcTOTPIcons = {};
 kpxcTOTPIcons.icons = [];
 
 kpxcTOTPIcons.newIcon = function(field, databaseState = DatabaseState.DISCONNECTED, segmented = false) {
@@ -160,6 +160,7 @@ TOTPFieldIcon.prototype.createIcon = function(field, segmented = false) {
 
     const styleSheet = createStylesheet('css/totp.css');
     const wrapper = document.createElement('div');
+    wrapper.style.all = 'unset';
     wrapper.style.display = 'none';
     styleSheet.addEventListener('load', () => wrapper.style.display = 'block');
 
