@@ -1,30 +1,11 @@
 'use strict';
 
+const options = {};
+
 const $ = function(elem) {
     return document.querySelector(elem);
 };
 
-(async() => {
-    try {
-        const settings = await browser.runtime.sendMessage({ action: 'load_settings' });
-        options.settings = settings;
-
-        const keyRing = await browser.runtime.sendMessage({ action: 'load_keyring' });
-        options.keyRing = keyRing;
-        options.initMenu();
-        options.initGeneralSettings();
-        options.initConnectedDatabases();
-        options.initCustomLoginFields();
-        options.initSitePreferences();
-        options.initAbout();
-        options.initTheme();
-    } catch (err) {
-        console.log('Error loading options page: ' + err);
-    }
-})();
-
-var options = options || {};
-
 options.initMenu = function() {
     const tabs = [].slice.call(document.querySelectorAll('div.tab'));
     const sideBarLinks = [].slice.call(document.querySelectorAll('.sidebar ul.nav li a'));
@@ -93,6 +74,8 @@ options.initGeneralSettings = function() {
         } else if (name === 'autoReconnect') {
             const message = updated.autoReconnect ? 'enable_automatic_reconnect' : 'disable_automatic_reconnect';
             browser.runtime.sendMessage({ action: message });
+        } else if (name === 'useMonochromeToolbarIcon') {
+            browser.runtime.sendMessage({ action: 'update_popup' });
         }
     };
 
@@ -101,11 +84,7 @@ options.initGeneralSettings = function() {
         options.saveSettings();
     };
 
-    if (options.settings['colorTheme'] === undefined) {
-        $('#tab-general-settings select#colorTheme').value = 'system';
-    } else {
-        $('#tab-general-settings select#colorTheme').value = options.settings['colorTheme'];
-    }
+    $('#tab-general-settings select#colorTheme').value = options.settings['colorTheme'];
 
     const generalSettingsCheckboxes = document.querySelectorAll('#tab-general-settings input[type=checkbox]');
     for (const checkbox of generalSettingsCheckboxes) {
@@ -149,7 +128,13 @@ options.initGeneralSettings = function() {
         // The theme is also stored in localStorage to prevent a white flash when the settings are first opened
         localStorage.setItem('colorTheme', options.settings['colorTheme']);
         await options.saveSettings();
-        location.reload();
+        options.updateTheme();
+    });
+
+    window.matchMedia('(prefers-color-scheme: dark)').addEventListener('change', () => {
+        if (options.settings['colorTheme'] === 'system') {
+            options.updateTheme();
+        }
     });
 
     $('#tab-general-settings select#credentialSorting').addEventListener('change', async function(e) {
@@ -231,11 +216,15 @@ options.initGeneralSettings = function() {
     });
 
     let temporarySettings;
-    const dialogImportSettingsModal = new bootstrap.Modal($('#dialogImportSettings'),
-        { keyboard: true, show: false, backdrop: true });
+    const dialogImportSettingsModal = new bootstrap.Modal('#dialogImportSettings',
+        { keyboard: true, focus: false, backdrop: true });
+
+    $('#dialogImportSettings').addEventListener('shown.bs.modal', function(modalEvent) {
+        modalEvent.currentTarget.querySelector('.modal-footer button.yes').focus();
+    });
 
     $('#importSettingsButton').addEventListener('click', function() {
-        var link = document.createElement('input');
+        const link = document.createElement('input');
         link.setAttribute('type', 'file');
         link.onchange = function(e) {
             const reader = new FileReader();
@@ -259,9 +248,6 @@ options.initGeneralSettings = function() {
                     // Verify the import
                     temporarySettings = contents;
                     dialogImportSettingsModal.show();
-                    $('#dialogImportSettings').addEventListener('shown.bs.modal', function(modalEvent) {
-                        modalEvent.currentTarget.querySelector('.modal-footer .btn-success').focus();
-                    });
                 } catch (err) {
                     console.log('Error loading JSON settings file.');
                 }
@@ -317,9 +303,9 @@ options.showKeePassXCVersions = async function(response) {
         response.latest = 'unknown';
     }
 
-    $('#tab-general-settings .kphVersion em.yourVersion').textContent = response.current;
-    $('#tab-general-settings .kphVersion em.latestVersion').textContent = response.latest;
-    $('#tab-about em.versionKPH').textContent = response.current;
+    $('#tab-general-settings .kphVersion span.yourVersion').textContent = response.current;
+    $('#tab-general-settings .kphVersion span.latestVersion').textContent = response.latest;
+    $('#tab-about span.versionKPH').textContent = response.current;
     $('#tab-about span.kpxcVersion').textContent = response.current;
     $('#tab-general-settings button.checkUpdateKeePassXC').disabled = false;
 
@@ -344,6 +330,16 @@ options.showKeePassXCVersions = async function(response) {
     if (!version270Result) {
         $('#tab-general-settings #downloadFaviconAfterSaveFormGroup').hide();
     }
+
+    // Hide certain options with older KeePassXC versions than 2.8.0
+    const version280Result = await browser.runtime.sendMessage({
+        action: 'compare_version',
+        args: [ '2.8.0', response.current ]
+    });
+
+    if (!version280Result) {
+        $('#tab-general-settings #passkeysOptionsCard').hide();
+    }
 };
 
 options.getPartiallyHiddenKey = function(key) {
@@ -351,16 +347,12 @@ options.getPartiallyHiddenKey = function(key) {
 };
 
 options.initConnectedDatabases = function() {
-    const dialogDeleteConnectedDatabaseModal = new bootstrap.Modal($('#dialogDeleteConnectedDatabase'),
-        { keyboard: true, show: false, backdrop: true });
+    const dialogDeleteConnectedDatabaseModal = new bootstrap.Modal('#dialogDeleteConnectedDatabase',
+        { keyboard: true, focus: false, backdrop: true });
 
-    const hideEmptyMessageRow = function() {
-        if ($('#tab-connected-databases table tbody').children.length > 2) {
-            $('#tab-connected-databases table tbody tr.empty').hide();
-        } else {
-            $('#tab-connected-databases table tbody tr.empty').style.display = '';
-        }
-    };
+    $('#dialogDeleteConnectedDatabase').addEventListener('shown.bs.modal', function(modalEvent) {
+        modalEvent.currentTarget.querySelector('.modal-footer button.yes').focus();
+    });
 
     $('#dialogDeleteConnectedDatabase .modal-footer button.yes').addEventListener('click', async function(e) {
         dialogDeleteConnectedDatabaseModal.hide();
@@ -377,7 +369,7 @@ options.initConnectedDatabases = function() {
             console.log(err);
         });
 
-        hideEmptyMessageRow();
+        browser.runtime.sendMessage({ action: 'update_popup' });
     });
 
     const removeButtonClicked = function(e) {
@@ -386,12 +378,13 @@ options.initConnectedDatabases = function() {
         const closestTr = this.closest('tr');
         $('#dialogDeleteConnectedDatabase').setAttribute('hash', closestTr.getAttribute('hash'));
         $('#dialogDeleteConnectedDatabase').setAttribute('tr-id', closestTr.getAttribute('id'));
-        $('#dialogDeleteConnectedDatabase .modal-body strong').textContent = closestTr.children[0].textContent;
+
+        const identifier = $('#dialogDeleteConnectedDatabase .modal-body strong');
+        if (identifier) {
+            identifier.textContent = closestTr.children[0].textContent;
+        }
 
         dialogDeleteConnectedDatabaseModal.show();
-        $('#dialogDeleteConnectedDatabase').addEventListener('shown.bs.modal', function(modalEvent) {
-            modalEvent.currentTarget.querySelector('.modal-footer .btn-success').focus();
-        });
     };
 
     const rowClone = $('#tab-connected-databases table tr.clone').cloneNode(true);
@@ -439,21 +432,15 @@ options.initConnectedDatabases = function() {
             }
         }
     });
-
-    hideEmptyMessageRow();
 };
 
 options.initCustomLoginFields = function() {
-    const dialogDeleteCustomLoginFieldsModal = new bootstrap.Modal($('#dialogDeleteCustomLoginFields'),
-        { keyboard: true, show: false, backdrop: true });
+    const dialogDeleteCustomLoginFieldsModal = new bootstrap.Modal('#dialogDeleteCustomLoginFields',
+        { keyboard: true, focus: false, backdrop: true });
 
-    const hideEmptyMessageRow = function() {
-        if ($('#tab-custom-fields table tbody').children.length > 2) {
-            $('#tab-custom-fields table tbody tr.empty').hide();
-        } else {
-            $('#tab-custom-fields table tbody tr.empty').style.display = '';
-        }
-    };
+    $('#dialogDeleteCustomLoginFields').addEventListener('shown.bs.modal', function(modalEvent) {
+        modalEvent.currentTarget.querySelector('.modal-footer button.yes').focus();
+    });
 
     const removeButtonClicked = function(e) {
         e.preventDefault();
@@ -464,9 +451,6 @@ options.initCustomLoginFields = function() {
         $('#dialogDeleteCustomLoginFields .modal-body strong').textContent = closestTr.children[0].textContent;
 
         dialogDeleteCustomLoginFieldsModal.show();
-        $('#dialogDeleteCustomLoginFields').addEventListener('shown.bs.modal', function(modalEvent) {
-            modalEvent.currentTarget.querySelector('.modal-footer .btn-success').focus();
-        });
     };
 
     $('#dialogDeleteCustomLoginFields .modal-footer button.yes').addEventListener('click', function(e) {
@@ -478,8 +462,6 @@ options.initCustomLoginFields = function() {
 
         delete options.settings['defined-custom-fields'][url];
         options.saveSettings();
-
-        hideEmptyMessageRow();
     });
 
     const rowClone = $('#tab-custom-fields table tr.clone').cloneNode(true);
@@ -496,8 +478,6 @@ options.initCustomLoginFields = function() {
         row.children[1].addEventListener('click', removeButtonClicked);
         $('#tab-custom-fields table tbody').append(row);
     }
-
-    hideEmptyMessageRow();
 };
 
 options.initSitePreferences = function() {
@@ -505,8 +485,12 @@ options.initSitePreferences = function() {
         options.settings['sitePreferences'] = [];
     }
 
-    const dialogDeleteSiteModal = new bootstrap.Modal($('#dialogDeleteSite'),
-        { keyboard: true, show: false, backdrop: true });
+    const dialogDeleteSiteModal = new bootstrap.Modal('#dialogDeleteSite',
+        { keyboard: true, focus: false, backdrop: true });
+
+    $('#dialogDeleteSite').addEventListener('shown.bs.modal', function(modalEvent) {
+        modalEvent.currentTarget.querySelector('.modal-footer button.yes').focus();
+    });
 
     const removeButtonClicked = function(e) {
         e.preventDefault();
@@ -517,9 +501,6 @@ options.initSitePreferences = function() {
         $('#dialogDeleteSite .modal-body strong').textContent = closestTr.children[0].textContent;
 
         dialogDeleteSiteModal.show();
-        $('#dialogDeleteSite').addEventListener('shown.bs.modal', function(modalEvent) {
-            modalEvent.currentTarget.querySelector('.modal-footer .btn-success').focus();
-        });
     };
 
     const checkboxClicked = function() {
@@ -552,14 +533,6 @@ options.initSitePreferences = function() {
         options.saveSettings();
     };
 
-    const hideEmptyMessageRow = function() {
-        if ($('#tab-site-preferences table tbody').children.length > 2) {
-            $('#tab-site-preferences table tbody tr.empty').hide();
-        } else {
-            $('#tab-site-preferences table tbody tr.empty').style.display = '';
-        }
-    };
-
     const addNewRow = function(rowClone, newIndex, url, ignore, usernameOnly, improvedFieldDetection) {
         const row = rowClone.cloneNode(true);
         row.setAttribute('url', url);
@@ -590,11 +563,6 @@ options.initSitePreferences = function() {
         }
 
         options.saveSettings();
-        hideEmptyMessageRow();
-    });
-
-    $('.was-validated').addEventListener('submit', function(e) {
-        e.preventDefault();
     });
 
     $('#manualUrl').addEventListener('keyup', function(event) {
@@ -653,8 +621,6 @@ options.initSitePreferences = function() {
             ++counter;
         }
     }
-
-    hideEmptyMessageRow();
 };
 
 options.initAbout = function() {
@@ -664,23 +630,19 @@ options.initAbout = function() {
         platform = 'Win64';
     }
 
-    $('#tab-about em.versionCIP').textContent = version;
+    $('#tab-about span.versionCIP').textContent = version;
     $('#tab-about span.kpxcbrVersion').textContent = version;
     $('#tab-about span.kpxcbrOS').textContent = platform;
     $('#tab-about span.kpxcbrBrowser').textContent = getBrowserId();
 };
 
-options.initTheme = function() {
-    if (options.settings['colorTheme'] === undefined) {
-        document.body.removeAttribute('data-color-theme');
-    } else {
-        document.body.setAttribute('data-color-theme', options.settings['colorTheme']);
-    }
-    // Sync localStorage setting
-    const localStorageTheme = localStorage.getItem('colorTheme');
-    if (localStorageTheme !== options.settings['colorTheme']) {
-        localStorage.setItem('colorTheme', options.settings['colorTheme']);
+options.updateTheme = function() {
+    let theme = options.settings['colorTheme'];
+    if (theme === 'system') {
+        theme = window.matchMedia('(prefers-color-scheme: dark)').matches ? 'dark' : 'light';
     }
+    document.documentElement.setAttribute('data-bs-theme', theme);
+    browser.runtime.sendMessage({ action: 'update_popup' });
 };
 
 options.createWarning = function(elem, text) {
@@ -716,3 +678,21 @@ const getBrowserId = function() {
 
     return 'Other/Unknown';
 };
+
+(async() => {
+    try {
+        const settings = await browser.runtime.sendMessage({ action: 'load_settings' });
+        options.settings = settings;
+
+        const keyRing = await browser.runtime.sendMessage({ action: 'load_keyring' });
+        options.keyRing = keyRing;
+        options.initMenu();
+        options.initGeneralSettings();
+        options.initConnectedDatabases();
+        options.initCustomLoginFields();
+        options.initSitePreferences();
+        options.initAbout();
+    } catch (err) {
+        console.log('Error loading options page: ' + err);
+    }
+})();
