@@ -19,6 +19,7 @@ function statusResponse(r) {
     $('#configured-not-associated').hide();
     $('#lock-database-button').hide();
     $('#getting-started-guide').hide();
+    $('#database-not-opened').hide();
 
     if (!r.keePassXCAvailable) {
         $('#error-message').textContent = r.error;
@@ -74,7 +75,6 @@ const sendMessageToTab = async function(message) {
 };
 
 (async () => {
-    resizePopup();
     await initColorTheme();
 
     $('#connect-button').addEventListener('click', async () => {
@@ -111,6 +111,7 @@ const sendMessageToTab = async function(message) {
             action: 'get_status',
             args: [ false, true ] // Set forcePopup to true
         }));
+        window.close();
     });
 
     $('#redetect-fields-button').addEventListener('click', async () => {
